<?php
/**
 * Implementation of the Custom Header feature.
 * http://codex.wordpress.org/Custom_Headers
 *
 * @uses admin/custom-background.php
 *
 * @package Ghostbird
 * @since 1.2
 */

/**
 * Setup the WordPress core custom header feature.
 *
 * Use add_theme_support to register support for WordPress 3.4+
 * as well as provide backward compatibility for previous versions.
 * Use feature detection of wp_get_theme() which was introduced
 * in WordPress 3.4.
 *
 * @uses ghostbird_header_style()
 * @uses ghostbird_admin_header_style()
 * @uses ghostbird_admin_header_image()
 *
 * @since Ghostbird 1.2
 */
function ghostbird_custom_header_setup() {
	$args = array(
		'default-image'          => get_template_directory_uri() . '/images/ghostbird.png',
		'default-text-color'     => 'ffffff',
		'width'                  => 240,
		'height'                 => 60,
		'flex-height'            => true,
		'flex-width'             => true,
		'wp-head-callback'       => 'ghostbird_header_style',
		'admin-head-callback'    => 'ghostbird_admin_header_style',
		'admin-preview-callback' => 'ghostbird_admin_header_image',
	);

	$args = apply_filters( 'ghostbird-custom-header-args', $args );

	if ( function_exists( 'wp_get_theme' ) ) {
		add_theme_support( 'custom-header', $args );
	} else {
		// Compat: Versions of WordPress prior to 3.4.
		define( 'HEADER_TEXTCOLOR',    $args['default-text-color'] );
		define( 'HEADER_IMAGE',        $args['default-image'] );
		define( 'HEADER_IMAGE_WIDTH',  $args['width'] );
		define( 'HEADER_IMAGE_HEIGHT', $args['height'] );
		add_custom_image_header( $args['wp-head-callback'], $args['admin-head-callback'], $args['admin-preview-callback'] );
	}
}
add_action( 'after_setup_theme', 'ghostbird_custom_header_setup' );

/**
 * Shiv for get_custom_header().
 *
 * get_custom_header() was introduced to WordPress
 * in version 3.4. To provide backward compatibility
 * with previous versions, we will define our own version
 * of this function.
 *
 * @return stdClass All properties represent attributes of the curent header image.
 *
 * @since Ghostbird 1.2
 */

if ( ! function_exists( 'get_custom_header' ) ) {
	function get_custom_header() {
		return (object) array(
			'url'           => get_header_image(),
			'thumbnail_url' => get_header_image(),
			'width'         => HEADER_IMAGE_WIDTH,
			'height'        => HEADER_IMAGE_HEIGHT,
		);
	}
}

if ( ! function_exists( 'ghostbird_header_style' ) ) :
/**
 * Styles the header image and text displayed on the blog
 *
 * @see ghostbird_custom_header_setup().
 *
 * @since Ghostbird 1.2
 */
function ghostbird_header_style() {

	// If no custom options for text are set, let's bail
	// get_header_textcolor() options: HEADER_TEXTCOLOR is default, hide text (returns 'blank') or any hex value
	if ( HEADER_TEXTCOLOR == get_header_textcolor() )
		return;

	// If we get this far, we have custom styles. Let's do this.
	?>
	<style type="text/css">
	<?php
		// Has the text been hidden?
		if ( 'blank' == get_header_textcolor() ) :
	?>
		#site-title,
		#tagline {
			position: absolute !important;
			clip: rect(1px 1px 1px 1px); /* IE6, IE7 */
			clip: rect(1px, 1px, 1px, 1px);
		}
	<?php
		// If the user has set a custom color for the text use that
		else :
	?>
		#site-title,
		#site-title a,
		#tagline {
			color: #<?php echo get_header_textcolor(); ?> !important;
		}
	<?php endif; ?>
	</style>
	<?php
}
endif; // ghostbird_header_style

if ( ! function_exists( 'ghostbird_admin_header_style' ) ) :
/**
 * Styles the header image displayed on the Appearance > Header admin panel.
 *
 * @see   ghostbird_custom_header_setup().
 * @uses  ghostbird_get_background_color().
 * @since Ghostbird 1.2
 */
function ghostbird_admin_header_style() {
?>
	<style type="text/css">
	.appearance_page_custom-header #headimg {
		<?php echo 'background-color: ' . esc_html( ghostbird_get_background_color() ) . ';'; ?>
		<?php if ( $url = get_background_image() )
			echo 'background-image: url( "' . esc_url( $url ) . '" );';
		?>
		width: 90%;
		padding: 30px;
		text-align: center;
	}
	#headimg h1,
	#headimg h1 a,
	#desc {
		color: #<?php echo get_header_textcolor(); ?> !important;
	}
	#headimg h1 {
		margin: 42px 0 14px 0;
		font-size: 44px;
		font-weight: bold;
		text-align: center;
	}
	#headimg h1 a {
		text-decoration: none;
	}
	#desc {
		font-size: 14px;
		font-style: italic;
		margin-top: 25px;
	}
	#headimg img {
		max-width: 700px;
		height: auto;
	}
	</style>
<?php
}
endif; // ghostbird_admin_header_style

if ( ! function_exists( 'ghostbird_admin_header_image' ) ) :
/**
 * Custom header image markup displayed on the Appearance > Header admin panel.
 *
 * @see ghostbird_custom_header_setup().
 *
 * @since Ghostbird 1.2
 */
function ghostbird_admin_header_image() { ?>
	<div id="headimg">
		<?php
		if ( 'blank' == get_theme_mod( 'header_textcolor', HEADER_TEXTCOLOR ) || '' == get_theme_mod( 'header_textcolor', HEADER_TEXTCOLOR ) )
			$style = ' style="display:none;"';
		else
			$style = ' style="color:#' . get_theme_mod( 'header_textcolor', HEADER_TEXTCOLOR ) . ';"';

		$header_image = get_header_image();

		if ( ! empty( $header_image ) ) : ?>
			<img src="<?php echo esc_url( $header_image ); ?>" alt="" />
			<h1><a id="name"<?php echo $style; ?> onclick="return false;" href="<?php echo esc_url( home_url( '/' ) ); ?>"><?php bloginfo( 'name' ); ?></a></h1>
			<div id="desc"<?php echo $style; ?>><?php bloginfo( 'description' ); ?></div>
		<?php endif; ?>
	</div>
<?php }
endif; // ghostbird_admin_header_image